import sys

from django.apps import AppConfig as DjangoAppConfig


class AppConfig(DjangoAppConfig):
    name = "edc_csf"
    verbose_name = "Edc CSF"
    has_exportable_data = True
    include_in_administration_section = True

    def ready(self):
        sys.stdout.write("Loading {} ...\n".format(self.verbose_name))
        sys.stdout.write(" Done loading {}.\n".format(self.verbose_name))



from typing import Any

from django import forms
from edc_constants.constants import NOT_DONE, YES
from edc_form_validators import REQUIRED_ERROR, FormValidator
from edc_lab.form_validators import CrfRequisitionFormValidatorMixin
from edc_visit_schedule.constants import DAY1

from .panels import csf_chemistry_panel, csf_panel


def validate_percentage(cleaned_data: dict, field: str, unit: str):
    if cleaned_data.get(field):
        if cleaned_data.get(unit) == "%" and cleaned_data.get(field) > 100:
            raise forms.ValidationError({field: "Cannot be greater than 100%."})


class LpFormValidatorMixin:
    def validate_lp(self: Any):
        self.required_if(
            YES, field="opening_pressure_measured", field_required="opening_pressure"
        )
        opening_pressure = self.cleaned_data.get("opening_pressure")
        closing_pressure = self.cleaned_data.get("closing_pressure")
        try:
            if opening_pressure <= closing_pressure:
                raise forms.ValidationError(
                    {"closing_pressure": "Cannot be greater than the opening pressure."}
                )
        except TypeError:
            pass


class CsfCultureFormValidatorMixin:
    def validate_csf_culture(self: Any):
        self.required_if(YES, field="csf_culture", field_required="other_csf_culture")

        self.require_together(
            field="csf_requisition",
            field_required="csf_assay_datetime",
        )

        self.validate_requisition("csf_requisition", "csf_assay_datetime", csf_chemistry_panel)

        self.require_together(
            field="differential_lymphocyte_count",
            field_required="differential_lymphocyte_unit",
        )

        validate_percentage(
            self.cleaned_data,
            field="differential_lymphocyte_count",
            unit="differential_lymphocyte_unit",
        )

        self.require_together(
            field="differential_neutrophil_count",
            field_required="differential_neutrophil_unit",
        )

        validate_percentage(
            self.cleaned_data,
            field="differential_neutrophil_count",
            unit="differential_neutrophil_unit",
        )

        # csf_glucose
        self.require_together(field="csf_glucose", field_required="csf_glucose_units")

        # csf_crag
        self.not_required_if(NOT_DONE, field="csf_crag", field_required="csf_crag_lfa")

        # csf_cr_ag and india_ink
        if (
            self.cleaned_data.get("subject_visit").visit_code == DAY1
            and self.cleaned_data.get("subject_visit").visit_code_sequence == 0
        ):
            if (
                self.cleaned_data.get("csf_crag") == NOT_DONE
                and self.cleaned_data.get("india_ink") == NOT_DONE
            ):
                error_msg = 'CSF CrAg and India Ink cannot both be "not done".'
                message = {"csf_crag": error_msg, "india_ink": error_msg}
                raise forms.ValidationError(message, code=REQUIRED_ERROR)


class QuantitativeCsfFormValidatorMixin:
    def validate_quantitative_culture(self: Any, requisition: str):
        self.require_together(
            field=requisition,
            field_required="qc_assay_datetime",
        )
        self.validate_requisition(requisition, "qc_assay_datetime", csf_panel)
        self.required_if_true(
            self.cleaned_data.get("quantitative_culture") is not None,
            field_required=requisition,
        )


class BiosynexSemiQuantitativeCragMixinFormValidatorMixin:
    def validate_biosynex_semi_quantitative_crag(self: Any):
        self.applicable_if(YES, field="bios_crag", field_applicable="crag_control_result")

        self.applicable_if(YES, field="bios_crag", field_applicable="crag_t1_result")

        self.applicable_if(YES, field="bios_crag", field_applicable="crag_t2_result")


class LpCsfFormValidatorMixin(
    CrfRequisitionFormValidatorMixin,
    LpFormValidatorMixin,
    QuantitativeCsfFormValidatorMixin,
    CsfCultureFormValidatorMixin,
):
    requisition_fields = [
        ("qc_requisition", "qc_assay_datetime"),
        ("csf_requisition", "csf_assay_datetime"),
    ]

    def clean(self):
        self.validate_lp()

        self.validate_quantitative_culture("qc_requisition")

        self.validate_csf_culture()


class LpCsfFormValidator(LpCsfFormValidatorMixin, FormValidator):
    pass



from decimal import Decimal

from django.core.exceptions import ValidationError
from django.test import TestCase
from edc_appointment.models import Appointment
from edc_constants.constants import NO, NOT_DONE, YES
from edc_reference import site_reference_configs
from edc_utils import get_utcnow
from edc_visit_schedule import site_visit_schedules
from edc_visit_tracking.constants import SCHEDULED
from visit_schedule_app.models import SubjectVisit
from visit_schedule_app.visit_schedule import visit_schedule

from edc_csf.form_validators import LpCsfFormValidator
from edc_csf.models import LpCsf
from edc_csf.panels import csf_panel
from edc_csf_app.models import Panel, SubjectRequisition


class TestLpFormValidator(TestCase):
    def setUp(self):
        site_visit_schedules._registry = {}
        site_visit_schedules.register(visit_schedule)
        site_reference_configs.register_from_visit_schedule(
            visit_models={"edc_appointment.appointment": "edc_visit_tracking.subjectvisit"}
        )
        self.subject_identifier = "1234"
        appointment = Appointment.objects.create(
            subject_identifier=self.subject_identifier,
            appt_datetime=get_utcnow(),
            visit_code="1000",
            visit_code_sequence=0,
            visit_schedule_name="visit_schedule",
            schedule_name="schedule",
            timepoint=Decimal("1.0"),
        )
        self.subject_visit = SubjectVisit.objects.create(
            appointment=appointment,
            report_datetime=appointment.appt_datetime,
            visit_code_sequence=0,
            visit_code="1000",
            reason=SCHEDULED,
            visit_schedule_name="visit_schedule",
            schedule_name="schedule",
        )

        appointment = Appointment.objects.create(
            subject_identifier=self.subject_identifier,
            appt_datetime=get_utcnow(),
            visit_code="3000",
            visit_code_sequence=0,
            visit_schedule_name="visit_schedule",
            schedule_name="schedule",
            timepoint=Decimal("1.0"),
        )
        self.subject_visit_d3 = SubjectVisit.objects.create(
            appointment=appointment,
            report_datetime=appointment.appt_datetime,
            visit_code_sequence=0,
            visit_code="3000",
            reason=SCHEDULED,
            visit_schedule_name="visit_schedule",
            schedule_name="schedule",
        )

    def test_pressure(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "opening_pressure": 10,
            "closing_pressure": 9,
        }
        form_validator = LpCsfFormValidator(cleaned_data=cleaned_data, instance=LpCsf())
        try:
            form_validator.validate()
        except ValidationError as e:
            self.fail(f"ValidationError unexpectedly raised. Got {e}")

        cleaned_data = {
            "subject_visit": self.subject_visit,
            "opening_pressure": 10,
            "closing_pressure": 11,
        }

        form_validator = LpCsfFormValidator(cleaned_data=cleaned_data, instance=LpCsf())
        self.assertRaises(ValidationError, form_validator.validate)
        self.assertIn("closing_pressure", form_validator._errors)
        self.assertIn("Cannot be greater", str(form_validator._errors.get("closing_pressure")))

    def test_other_csf_culture_required(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "csf_culture": YES,
            "other_csf_culture": None,
        }
        form_validator = LpCsfFormValidator(cleaned_data=cleaned_data, instance=LpCsf())
        self.assertRaises(ValidationError, form_validator.validate)
        self.assertIn("other_csf_culture", form_validator._errors)
        self.assertIn("is required", str(form_validator._errors.get("other_csf_culture")))

    def test_other_csf_culture_not_required(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "csf_culture": NO,
            "other_csf_culture": "blah",
        }
        form_validator = LpCsfFormValidator(cleaned_data=cleaned_data, instance=LpCsf())
        self.assertRaises(ValidationError, form_validator.validate)
        self.assertIn("other_csf_culture", form_validator._errors)
        self.assertIn("not required", str(form_validator._errors.get("other_csf_culture")))

    def test_csf_culture_not_perfomed(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "csf_culture": NOT_DONE,
            "other_csf_culture": "culture",
        }
        form_validator = LpCsfFormValidator(cleaned_data=cleaned_data, instance=LpCsf())
        self.assertRaises(ValidationError, form_validator.validate)
        self.assertIn("other_csf_culture", form_validator._errors)
        self.assertIn("not required", str(form_validator._errors.get("other_csf_culture")))

    def test_qc_culture(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "qc_requisition": None,
            "qc_assay_datetime": None,
            "quantitative_culture": None,
        }
        form_validator = LpCsfFormValidator(cleaned_data=cleaned_data, instance=LpCsf())
        try:
            form_validator.validate()
        except ValidationError:
            self.fail("ValidationError unexpectedly raised")

    def test_qc_culture_requisition_not_required_if_no_result(self):
        panel = Panel.objects.create(name=csf_panel.name)
        requisition = SubjectRequisition.objects.create(
            subject_visit=self.subject_visit, panel=panel
        )
        requisition.panel_object = csf_panel
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "qc_requisition": requisition,
            "qc_assay_datetime": get_utcnow(),
            "quantitative_culture": None,
        }
        form_validator = LpCsfFormValidator(cleaned_data=cleaned_data, instance=LpCsf())
        self.assertRaises(ValidationError, form_validator.validate)
        self.assertIn(
            "This field is not required",
            str(form_validator._errors.get("qc_requisition")),
        )

    def test_qc_assay_datetime_not_required_if_no_result(self):
        panel = Panel.objects.create(name=csf_panel.name)
        requisition = SubjectRequisition.objects.create(
            subject_visit=self.subject_visit, panel=panel
        )
        requisition.panel_object = csf_panel
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "qc_requisition": None,
            "qc_assay_datetime": get_utcnow(),
            "quantitative_culture": None,
        }
        form_validator = LpCsfFormValidator(cleaned_data=cleaned_data, instance=LpCsf())
        self.assertRaises(ValidationError, form_validator.validate)
        self.assertIn(
            "This field is not required",
            str(form_validator._errors.get("qc_assay_datetime")),
        )

    def test_qc_culture_requisition_required_if_value(self):
        """Requisition is required if there is a value."""
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "qc_requisition": None,
            "qc_assay_datetime": None,
            "quantitative_culture": "12",
        }
        form_validator = LpCsfFormValidator(cleaned_data=cleaned_data, instance=LpCsf())
        self.assertRaises(ValidationError, form_validator.validate)
        self.assertIn(
            "This field is required", str(form_validator._errors.get("qc_requisition"))
        )

    def test_qc_assay_datetime_required_if_value(self):
        """Requisition is required if there is a value."""
        panel = Panel.objects.create(name=csf_panel.name)
        requisition = SubjectRequisition.objects.create(
            subject_visit=self.subject_visit, panel=panel
        )
        requisition.panel_object = csf_panel
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "qc_requisition": requisition,
            "qc_assay_datetime": None,
            "quantitative_culture": "12",
        }
        form_validator = LpCsfFormValidator(cleaned_data=cleaned_data, instance=LpCsf())
        self.assertRaises(ValidationError, form_validator.validate)
        self.assertIn(
            "This field is required",
            str(form_validator._errors.get("qc_assay_datetime")),
        )

    def test_india_ink_csf_arg_not_done_invalid(self):
        """Assert that either csf_cr_ag or india_ink is done."""
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "csf_crag": NOT_DONE,
            "india_ink": NOT_DONE,
        }
        form_validator = LpCsfFormValidator(cleaned_data=cleaned_data, instance=LpCsf())
        self.assertRaises(ValidationError, form_validator.validate)
        self.assertIn("csf_crag", form_validator._errors)
        self.assertIn("india_ink", form_validator._errors)

    def test_csf_wbc_cell_count_not_required_day1(self):
        cleaned_data = {"subject_visit": self.subject_visit, "csf_wbc_cell_count": None}
        form_validator = LpCsfFormValidator(cleaned_data=cleaned_data, instance=LpCsf())
        try:
            form_validator.validate()
        except ValidationError:
            self.fail("ValidationError unexpectedly raised")

    def test_csf_wbc_cell_count_not_required_day3(self):
        cleaned_data = {
            "subject_visit": self.subject_visit_d3,
            "csf_wbc_cell_count": None,
        }
        form_validator = LpCsfFormValidator(cleaned_data=cleaned_data, instance=LpCsf())
        try:
            form_validator.validate()
        except ValidationError:
            self.fail("ValidationError unexpectedly raised")

    def test_csf_wbc_cell_count_min_max(self):
        for i in range(0, 5):
            cleaned_data = {
                "subject_visit": self.subject_visit_d3,
                "csf_wbc_cell_count": i,
            }
            form_validator = LpCsfFormValidator(cleaned_data=cleaned_data, instance=LpCsf())
            try:
                form_validator.validate()
            except ValidationError:
                self.fail(
                    f"ValidationError unexpectedly raised. " f"Got {form_validator._errors}"
                )

    def test_csf_crag_not_done_csf_crag_lfa_not_required(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "csf_crag": NOT_DONE,
            "csf_crag_lfa": YES,
        }
        form_validator = LpCsfFormValidator(cleaned_data=cleaned_data, instance=LpCsf())
        self.assertRaises(ValidationError, form_validator.validate)
        self.assertIn("csf_crag_lfa", form_validator._errors)
        self.assertIn("not required", str(form_validator._errors.get("csf_crag_lfa")))

    def test_differential_neutrophil_count_percent_limit_passed(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "csf_wbc_cell_count": 4,
            "differential_lymphocyte_count": 50,
            "differential_lymphocyte_unit": "%",
            "differential_neutrophil_count": 125.6,
            "differential_neutrophil_unit": "%",
        }
        form_validator = LpCsfFormValidator(cleaned_data=cleaned_data, instance=LpCsf())
        self.assertRaises(ValidationError, form_validator.validate)
        self.assertIn("differential_neutrophil_count", form_validator._errors)
        self.assertIn(
            "Cannot be greater than 100%",
            str(form_validator._errors.get("differential_neutrophil_count")),
        )

    def test_differential_lymphocyte_count_percent_limit_passed(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "csf_wbc_cell_count": 4,
            "differential_lymphocyte_count": 125.6,
            "differential_lymphocyte_unit": "%",
        }
        form_validator = LpCsfFormValidator(cleaned_data=cleaned_data, instance=LpCsf())
        self.assertRaises(ValidationError, form_validator.validate)
        self.assertIn("differential_lymphocyte_count", form_validator._errors)
        self.assertIn(
            "Cannot be greater than 100%",
            str(form_validator._errors.get("differential_lymphocyte_count")),
        )



from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from edc_constants.choices import (
    HIGH_LOW_NA,
    POS_NEG_NOT_DONE_NA,
    YES_NO_AWAITING_RESULTS,
    YES_NO_NA,
)
from edc_constants.constants import AWAITING_RESULTS, NOT_APPLICABLE


class CsfModelMixin(models.Model):
    csf_positive = models.CharField(
        verbose_name="CSF positive for cryptococcal meningitis?",
        max_length=18,
        choices=YES_NO_AWAITING_RESULTS,
        default=AWAITING_RESULTS,
    )

    india_ink = models.CharField(
        max_length=15, choices=POS_NEG_NOT_DONE_NA, default=NOT_APPLICABLE
    )

    csf_crag_lfa = models.CharField(
        verbose_name="CrAg LFA",
        max_length=15,
        choices=POS_NEG_NOT_DONE_NA,
        default=NOT_APPLICABLE,
    )

    sq_crag = models.CharField(
        verbose_name="SQ CrAg",
        max_length=15,
        choices=POS_NEG_NOT_DONE_NA,
        default=NOT_APPLICABLE,
    )

    sq_crag_pos = models.CharField(
        verbose_name="SQ CrAg",
        max_length=15,
        choices=HIGH_LOW_NA,
        default=NOT_APPLICABLE,
    )

    crf_crag_titre_done = models.CharField(
        verbose_name="Was the CRF CrAg titre done",
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
    )
    crf_crag_titre = models.IntegerField(
        validators=[MinValueValidator(1), MaxValueValidator(999)],
        null=True,
        blank=True,
    )

    class Meta:
        abstract = True



from django.contrib import admin
from django_audit_fields import audit_fieldset_tuple

from .fieldsets import (
    get_csf_culture_fieldset,
    get_lp_fieldset,
    get_quantitative_culture_fieldset,
)


class LpCsfModelAdminMixin:
    fieldsets = (
        get_lp_fieldset(),
        get_csf_culture_fieldset(),
        get_quantitative_culture_fieldset(),
        audit_fieldset_tuple,
    )

    radio_fields = {
        "reason_for_lp": admin.VERTICAL,
        "csf_culture": admin.VERTICAL,
        "india_ink": admin.VERTICAL,
        "csf_crag": admin.VERTICAL,
        "csf_crag_lfa": admin.VERTICAL,
        "differential_lymphocyte_unit": admin.VERTICAL,
        "differential_neutrophil_unit": admin.VERTICAL,
        "csf_glucose_units": admin.VERTICAL,
        "bios_crag": admin.VERTICAL,
        "crag_control_result": admin.VERTICAL,
        "crag_t1_result": admin.VERTICAL,
        "crag_t2_result": admin.VERTICAL,
    }

    list_display = ("lp_datetime", "reason_for_lp")

    list_filter = ("lp_datetime", "reason_for_lp")



# Generated by Django 4.1.2 on 2022-12-09 18:33

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_csf", "0007_auto_20220711_1230"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicallpcsf",
            name="csf_amount_removed",
            field=models.IntegerField(
                blank=True,
                help_text="Units ml",
                null=True,
                validators=[django.core.validators.MinValueValidator(1)],
                verbose_name="CSF amount removed",
            ),
        ),
        migrations.AlterField(
            model_name="lpcsf",
            name="csf_amount_removed",
            field=models.IntegerField(
                blank=True,
                help_text="Units ml",
                null=True,
                validators=[django.core.validators.MinValueValidator(1)],
                verbose_name="CSF amount removed",
            ),
        ),
    ]



from django.db import models
from django.db.models import PROTECT
from edc_crf.model_mixins import CrfModelMixin
from edc_identifier.model_mixins import NonUniqueSubjectIdentifierFieldMixin
from edc_lab.models import Panel
from edc_model.models import BaseUuidModel
from edc_visit_schedule.model_mixins import VisitCodeFieldsModelMixin
from visit_schedule_app.models import SubjectVisit


class SubjectRequisition(
    NonUniqueSubjectIdentifierFieldMixin, VisitCodeFieldsModelMixin, BaseUuidModel
):
    subject_visit = models.ForeignKey(SubjectVisit, on_delete=models.PROTECT, related_name="+")

    panel = models.ForeignKey(Panel, on_delete=PROTECT)

    class Meta(BaseUuidModel.Meta):
        pass


class Crf(CrfModelMixin, BaseUuidModel):
    subject_visit = models.ForeignKey(SubjectVisit, on_delete=PROTECT)

    f1 = models.CharField(max_length=50, null=True)

    f2 = models.CharField(max_length=50, null=True)

    f3 = models.CharField(max_length=50, null=True)



# Generated by Django 3.2.11 on 2022-02-25 02:00

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_csf", "0003_auto_20220225_0423"),
    ]

    operations = [
        migrations.RenameField(
            model_name="historicallpcsf",
            old_name="csf_assay_datetime",
            new_name="csf_culture_assay_datetime",
        ),
        migrations.RenameField(
            model_name="lpcsf",
            old_name="csf_assay_datetime",
            new_name="csf_culture_assay_datetime",
        ),
        migrations.AddField(
            model_name="historicallpcsf",
            name="crag_lfa",
            field=models.CharField(
                choices=[
                    ("POS", "Positive"),
                    ("NEG", "Negative"),
                    ("not_done", "Not done"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
                verbose_name="CrAg LFA",
            ),
        ),
        migrations.AddField(
            model_name="historicallpcsf",
            name="crf_crag_titre",
            field=models.IntegerField(
                blank=True,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(999),
                ],
            ),
        ),
        migrations.AddField(
            model_name="historicallpcsf",
            name="crf_crag_titre_done",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=15,
                verbose_name="Was the CRF CrAg titre done",
            ),
        ),
        migrations.AddField(
            model_name="historicallpcsf",
            name="csf_positive",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("PENDING", "Pending"),
                    ("N/A", "Not applicable"),
                ],
                default="PENDING",
                help_text="Complete after getting the results.",
                max_length=18,
                verbose_name="CSF positive for cryptococcal meningitis?",
            ),
        ),
        migrations.AddField(
            model_name="historicallpcsf",
            name="csf_results_available",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="Yes",
                max_length=15,
                verbose_name="Are CSF results available?",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="historicallpcsf",
            name="sq_crag",
            field=models.CharField(
                choices=[
                    ("POS", "Positive"),
                    ("NEG", "Negative"),
                    ("not_done", "Not done"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
                verbose_name="SQ CrAg",
            ),
        ),
        migrations.AddField(
            model_name="historicallpcsf",
            name="sq_crag_pos",
            field=models.CharField(
                choices=[("high", "High"), ("low", "Low"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=15,
                verbose_name="SQ CrAg",
            ),
        ),
        migrations.AddField(
            model_name="lpcsf",
            name="crag_lfa",
            field=models.CharField(
                choices=[
                    ("POS", "Positive"),
                    ("NEG", "Negative"),
                    ("not_done", "Not done"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
                verbose_name="CrAg LFA",
            ),
        ),
        migrations.AddField(
            model_name="lpcsf",
            name="crf_crag_titre",
            field=models.IntegerField(
                blank=True,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(999),
                ],
            ),
        ),
        migrations.AddField(
            model_name="lpcsf",
            name="crf_crag_titre_done",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=15,
                verbose_name="Was the CRF CrAg titre done",
            ),
        ),
        migrations.AddField(
            model_name="lpcsf",
            name="csf_positive",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("PENDING", "Pending"),
                    ("N/A", "Not applicable"),
                ],
                default="PENDING",
                help_text="Complete after getting the results.",
                max_length=18,
                verbose_name="CSF positive for cryptococcal meningitis?",
            ),
        ),
        migrations.AddField(
            model_name="lpcsf",
            name="csf_results_available",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="Yes",
                max_length=15,
                verbose_name="Are CSF results available?",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="lpcsf",
            name="sq_crag",
            field=models.CharField(
                choices=[
                    ("POS", "Positive"),
                    ("NEG", "Negative"),
                    ("not_done", "Not done"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
                verbose_name="SQ CrAg",
            ),
        ),
        migrations.AddField(
            model_name="lpcsf",
            name="sq_crag_pos",
            field=models.CharField(
                choices=[("high", "High"), ("low", "Low"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=15,
                verbose_name="SQ CrAg",
            ),
        ),
        migrations.AlterField(
            model_name="historicallpcsf",
            name="india_ink",
            field=models.CharField(
                choices=[
                    ("POS", "Positive"),
                    ("NEG", "Negative"),
                    ("not_done", "Not done"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
            ),
        ),
        migrations.AlterField(
            model_name="lpcsf",
            name="india_ink",
            field=models.CharField(
                choices=[
                    ("POS", "Positive"),
                    ("NEG", "Negative"),
                    ("not_done", "Not done"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
            ),
        ),
    ]



from edc_lab import Process, ProcessingProfile

from .aliquot_types import (
    csf,
    csf_glucose,
    csf_pellet,
    csf_supernatant,
    csf_testing,
    qfc,
)

csf_store_processing_profile = ProcessingProfile(name="csf_culture", aliquot_type=csf)
process_qfc = Process(aliquot_type=qfc, aliquot_count=3)
process_csf_testing = Process(aliquot_type=csf_testing, aliquot_count=2)
csf_store_processing_profile.add_processes(process_qfc, process_csf_testing)

csf_stop_processing_profile = ProcessingProfile(name="csf_store", aliquot_type=csf)
csf_store = Process(aliquot_type=qfc, aliquot_count=2)
process_csf_testing = Process(aliquot_type=csf_testing, aliquot_count=1)
csf_stop_processing_profile.add_processes(csf_store, process_csf_testing)

csf_pkpd_processing_profile = ProcessingProfile(name="csf_pkpd", aliquot_type=csf)
process_csf_pkpd = Process(aliquot_type=csf, aliquot_count=2)
csf_pkpd_processing_profile.add_processes(process_csf_pkpd)

qpcr_csf_processing_profile = ProcessingProfile(name="qpcr_csf", aliquot_type=csf)
process_supernatant = Process(aliquot_type=csf_supernatant, aliquot_count=1)
process_pellet = Process(aliquot_type=csf_pellet, aliquot_count=1)
qpcr_csf_processing_profile.add_processes(process_supernatant, process_pellet)

csf_chem_processing_profile = ProcessingProfile(name="csf_chemistry", aliquot_type=csf)
process_csf_glucose = Process(aliquot_type=csf_glucose, aliquot_count=1)



# Generated by Django 3.2.11 on 2022-02-22 04:21

import uuid

import _socket
import django.core.validators
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_revision.revision_field
import edc_model.validators.date
import edc_sites.models
import edc_utils.date
import edc_visit_tracking.managers
import simple_history.models
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("sites", "0002_alter_domain_unique"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="LpCsf",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("subject_identifier", models.CharField(max_length=50, unique=True)),
                (
                    "bios_crag",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        help_text="Gaborone and Blantyre only",
                        max_length=5,
                        verbose_name="Biosynex Semi-quantitative CrAg performed?",
                    ),
                ),
                (
                    "crag_control_result",
                    models.CharField(
                        choices=[
                            ("POS", "Positive"),
                            ("NEG", "Negative"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        help_text="Gaborone and Blantyre only",
                        max_length=5,
                        verbose_name="Control result",
                    ),
                ),
                (
                    "crag_t1_result",
                    models.CharField(
                        choices=[
                            ("POS", "Positive"),
                            ("NEG", "Negative"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        help_text="Gaborone and Blantyre only",
                        max_length=5,
                        verbose_name="T1 result",
                    ),
                ),
                (
                    "crag_t2_result",
                    models.CharField(
                        choices=[
                            ("POS", "Positive"),
                            ("NEG", "Negative"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        help_text="Gaborone and Blantyre only",
                        max_length=5,
                        verbose_name="T2 result",
                    ),
                ),
                (
                    "csf_assay_datetime",
                    models.DateTimeField(
                        blank=True,
                        null=True,
                        validators=[edc_model.validators.date.datetime_not_future],
                        verbose_name="CSF Result Report Date and Time",
                    ),
                ),
                (
                    "csf_culture",
                    models.CharField(
                        choices=[
                            ("Yes", "Yes"),
                            ("No", "No"),
                            ("awaiting_results", "Awaiting results"),
                            ("not_done", "Not done"),
                        ],
                        default="awaiting_results",
                        help_text="Complete after getting the results.",
                        max_length=18,
                        verbose_name="Other organism (non-Cryptococcus)",
                    ),
                ),
                (
                    "other_csf_culture",
                    models.CharField(
                        blank=True,
                        max_length=75,
                        null=True,
                        verbose_name="If YES, specify organism:",
                    ),
                ),
                (
                    "csf_wbc_cell_count",
                    models.IntegerField(
                        blank=True,
                        help_text="acceptable units are mm<sup>3</sup>",
                        null=True,
                        validators=[django.core.validators.MinValueValidator(0)],
                        verbose_name="Total CSF WBC cell count:",
                    ),
                ),
                (
                    "differential_lymphocyte_count",
                    models.IntegerField(
                        blank=True,
                        help_text="acceptable units are mm<sup>3</sup> or %",
                        null=True,
                        validators=[django.core.validators.MinValueValidator(0)],
                        verbose_name="Differential lymphocyte cell count:",
                    ),
                ),
                (
                    "differential_lymphocyte_unit",
                    models.CharField(
                        blank=True,
                        choices=[("mm3", "mm<sup>3</sup>"), ("%", "%")],
                        max_length=6,
                        null=True,
                    ),
                ),
                (
                    "differential_neutrophil_count",
                    models.IntegerField(
                        blank=True,
                        help_text="acceptable units are mm<sup>3</sup> or %",
                        null=True,
                        validators=[django.core.validators.MinValueValidator(0)],
                        verbose_name="Differential neutrophil cell count:",
                    ),
                ),
                (
                    "differential_neutrophil_unit",
                    models.CharField(
                        blank=True,
                        choices=[("mm3", "mm<sup>3</sup>"), ("%", "%")],
                        max_length=6,
                        null=True,
                    ),
                ),
                (
                    "india_ink",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("POS", "Positive"),
                            ("NEG", "Negative"),
                            ("IND", "Indeterminate"),
                        ],
                        max_length=15,
                        null=True,
                    ),
                ),
                (
                    "csf_glucose",
                    models.DecimalField(
                        blank=True,
                        decimal_places=1,
                        help_text="Units in mmol/L or mg/dL",
                        max_digits=3,
                        null=True,
                        verbose_name="CSF glucose:",
                    ),
                ),
                (
                    "csf_glucose_units",
                    models.CharField(
                        blank=True,
                        choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                        max_length=6,
                        null=True,
                        verbose_name="CSF glucose units:",
                    ),
                ),
                (
                    "csf_protein",
                    models.DecimalField(
                        blank=True,
                        decimal_places=2,
                        help_text="Units in g/L",
                        max_digits=4,
                        null=True,
                        verbose_name="CSF protein:",
                    ),
                ),
                (
                    "csf_cr_ag",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("POS", "Positive"),
                            ("NEG", "Negative"),
                            ("IND", "Indeterminate"),
                        ],
                        max_length=15,
                        null=True,
                        verbose_name="CSF CrAg:",
                    ),
                ),
                (
                    "csf_cr_ag_lfa",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        max_length=5,
                        verbose_name="CSF CrAg done by IMMY CrAg LFA:",
                    ),
                ),
                (
                    "qc_assay_datetime",
                    models.DateTimeField(
                        blank=True,
                        null=True,
                        validators=[edc_model.validators.date.datetime_not_future],
                        verbose_name="QC Result Report Date and Time",
                    ),
                ),
                (
                    "quantitative_culture",
                    models.IntegerField(
                        blank=True,
                        help_text="Units CFU/ml",
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(0),
                            django.core.validators.MaxValueValidator(100000000),
                        ],
                    ),
                ),
                (
                    "lp_datetime",
                    models.DateTimeField(
                        validators=[edc_model.validators.date.datetime_not_future],
                        verbose_name="LP Date and Time",
                    ),
                ),
                (
                    "reason_for_lp",
                    models.CharField(
                        choices=[
                            ("scheduled_per_protocol", "Scheduled per protocol"),
                            ("therapeutic_lp", "Therapeutic LP"),
                            ("clincal_deterioration", "Clinical deterioration"),
                        ],
                        max_length=50,
                        verbose_name="Reason for LP",
                    ),
                ),
                (
                    "opening_pressure",
                    models.IntegerField(
                        help_text="Units cm of H<sub>2</sub>O",
                        validators=[
                            django.core.validators.MinValueValidator(1),
                            django.core.validators.MaxValueValidator(99),
                        ],
                    ),
                ),
                (
                    "closing_pressure",
                    models.IntegerField(
                        blank=True,
                        help_text="Units cm of H<sub>2</sub>O",
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(0),
                            django.core.validators.MaxValueValidator(99),
                        ],
                    ),
                ),
                (
                    "csf_amount_removed",
                    models.IntegerField(
                        blank=True,
                        help_text="Units ml",
                        null=True,
                        validators=[django.core.validators.MinValueValidator(1)],
                        verbose_name="CSF amount removed ",
                    ),
                ),
                ("report_datetime", models.DateTimeField(default=edc_utils.date.get_utcnow)),
                (
                    "site",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
            ],
            options={
                "verbose_name": "Lumbar Puncture/Cerebrospinal Fluid",
                "verbose_name_plural": "Lumbar Puncture/Cerebrospinal Fluid",
                "ordering": ("-modified", "-created"),
                "get_latest_by": "modified",
                "abstract": False,
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
            },
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
            ],
        ),
        migrations.CreateModel(
            name="HistoricalLpCsf",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                ("subject_identifier", models.CharField(db_index=True, max_length=50)),
                (
                    "bios_crag",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        help_text="Gaborone and Blantyre only",
                        max_length=5,
                        verbose_name="Biosynex Semi-quantitative CrAg performed?",
                    ),
                ),
                (
                    "crag_control_result",
                    models.CharField(
                        choices=[
                            ("POS", "Positive"),
                            ("NEG", "Negative"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        help_text="Gaborone and Blantyre only",
                        max_length=5,
                        verbose_name="Control result",
                    ),
                ),
                (
                    "crag_t1_result",
                    models.CharField(
                        choices=[
                            ("POS", "Positive"),
                            ("NEG", "Negative"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        help_text="Gaborone and Blantyre only",
                        max_length=5,
                        verbose_name="T1 result",
                    ),
                ),
                (
                    "crag_t2_result",
                    models.CharField(
                        choices=[
                            ("POS", "Positive"),
                            ("NEG", "Negative"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        help_text="Gaborone and Blantyre only",
                        max_length=5,
                        verbose_name="T2 result",
                    ),
                ),
                (
                    "csf_assay_datetime",
                    models.DateTimeField(
                        blank=True,
                        null=True,
                        validators=[edc_model.validators.date.datetime_not_future],
                        verbose_name="CSF Result Report Date and Time",
                    ),
                ),
                (
                    "csf_culture",
                    models.CharField(
                        choices=[
                            ("Yes", "Yes"),
                            ("No", "No"),
                            ("awaiting_results", "Awaiting results"),
                            ("not_done", "Not done"),
                        ],
                        default="awaiting_results",
                        help_text="Complete after getting the results.",
                        max_length=18,
                        verbose_name="Other organism (non-Cryptococcus)",
                    ),
                ),
                (
                    "other_csf_culture",
                    models.CharField(
                        blank=True,
                        max_length=75,
                        null=True,
                        verbose_name="If YES, specify organism:",
                    ),
                ),
                (
                    "csf_wbc_cell_count",
                    models.IntegerField(
                        blank=True,
                        help_text="acceptable units are mm<sup>3</sup>",
                        null=True,
                        validators=[django.core.validators.MinValueValidator(0)],
                        verbose_name="Total CSF WBC cell count:",
                    ),
                ),
                (
                    "differential_lymphocyte_count",
                    models.IntegerField(
                        blank=True,
                        help_text="acceptable units are mm<sup>3</sup> or %",
                        null=True,
                        validators=[django.core.validators.MinValueValidator(0)],
                        verbose_name="Differential lymphocyte cell count:",
                    ),
                ),
                (
                    "differential_lymphocyte_unit",
                    models.CharField(
                        blank=True,
                        choices=[("mm3", "mm<sup>3</sup>"), ("%", "%")],
                        max_length=6,
                        null=True,
                    ),
                ),
                (
                    "differential_neutrophil_count",
                    models.IntegerField(
                        blank=True,
                        help_text="acceptable units are mm<sup>3</sup> or %",
                        null=True,
                        validators=[django.core.validators.MinValueValidator(0)],
                        verbose_name="Differential neutrophil cell count:",
                    ),
                ),
                (
                    "differential_neutrophil_unit",
                    models.CharField(
                        blank=True,
                        choices=[("mm3", "mm<sup>3</sup>"), ("%", "%")],
                        max_length=6,
                        null=True,
                    ),
                ),
                (
                    "india_ink",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("POS", "Positive"),
                            ("NEG", "Negative"),
                            ("IND", "Indeterminate"),
                        ],
                        max_length=15,
                        null=True,
                    ),
                ),
                (
                    "csf_glucose",
                    models.DecimalField(
                        blank=True,
                        decimal_places=1,
                        help_text="Units in mmol/L or mg/dL",
                        max_digits=3,
                        null=True,
                        verbose_name="CSF glucose:",
                    ),
                ),
                (
                    "csf_glucose_units",
                    models.CharField(
                        blank=True,
                        choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                        max_length=6,
                        null=True,
                        verbose_name="CSF glucose units:",
                    ),
                ),
                (
                    "csf_protein",
                    models.DecimalField(
                        blank=True,
                        decimal_places=2,
                        help_text="Units in g/L",
                        max_digits=4,
                        null=True,
                        verbose_name="CSF protein:",
                    ),
                ),
                (
                    "csf_cr_ag",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("POS", "Positive"),
                            ("NEG", "Negative"),
                            ("IND", "Indeterminate"),
                        ],
                        max_length=15,
                        null=True,
                        verbose_name="CSF CrAg:",
                    ),
                ),
                (
                    "csf_cr_ag_lfa",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        max_length=5,
                        verbose_name="CSF CrAg done by IMMY CrAg LFA:",
                    ),
                ),
                (
                    "qc_assay_datetime",
                    models.DateTimeField(
                        blank=True,
                        null=True,
                        validators=[edc_model.validators.date.datetime_not_future],
                        verbose_name="QC Result Report Date and Time",
                    ),
                ),
                (
                    "quantitative_culture",
                    models.IntegerField(
                        blank=True,
                        help_text="Units CFU/ml",
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(0),
                            django.core.validators.MaxValueValidator(100000000),
                        ],
                    ),
                ),
                (
                    "lp_datetime",
                    models.DateTimeField(
                        validators=[edc_model.validators.date.datetime_not_future],
                        verbose_name="LP Date and Time",
                    ),
                ),
                (
                    "reason_for_lp",
                    models.CharField(
                        choices=[
                            ("scheduled_per_protocol", "Scheduled per protocol"),
                            ("therapeutic_lp", "Therapeutic LP"),
                            ("clincal_deterioration", "Clinical deterioration"),
                        ],
                        max_length=50,
                        verbose_name="Reason for LP",
                    ),
                ),
                (
                    "opening_pressure",
                    models.IntegerField(
                        help_text="Units cm of H<sub>2</sub>O",
                        validators=[
                            django.core.validators.MinValueValidator(1),
                            django.core.validators.MaxValueValidator(99),
                        ],
                    ),
                ),
                (
                    "closing_pressure",
                    models.IntegerField(
                        blank=True,
                        help_text="Units cm of H<sub>2</sub>O",
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(0),
                            django.core.validators.MaxValueValidator(99),
                        ],
                    ),
                ),
                (
                    "csf_amount_removed",
                    models.IntegerField(
                        blank=True,
                        help_text="Units ml",
                        null=True,
                        validators=[django.core.validators.MinValueValidator(1)],
                        verbose_name="CSF amount removed ",
                    ),
                ),
                ("report_datetime", models.DateTimeField(default=edc_utils.date.get_utcnow)),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4, editable=False, primary_key=True, serialize=False
                    ),
                ),
                ("history_date", models.DateTimeField()),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
            ],
            options={
                "verbose_name": "historical Lumbar Puncture/Cerebrospinal Fluid",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": "history_date",
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
    ]



from edc_constants.disease_constants import THERAPEUTIC_LP
from edc_reportable import (
    MILLIGRAMS_PER_DECILITER,
    MILLIMOLES_PER_LITER,
    MILLIMOLES_PER_LITER_DISPLAY,
    MM3,
    MM3_DISPLAY,
)

LP_REASON = (
    ("scheduled_per_protocol", "Scheduled per protocol"),
    (THERAPEUTIC_LP, "Therapeutic LP"),
    ("clincal_deterioration", "Clinical deterioration"),
)

MG_MMOL_UNITS = (
    (MILLIGRAMS_PER_DECILITER, MILLIGRAMS_PER_DECILITER),
    (MILLIMOLES_PER_LITER, MILLIMOLES_PER_LITER_DISPLAY),
)


MM3_PERC_UNITS = ((MM3, MM3_DISPLAY), ("%", "%"))



from django.db import models
from edc_identifier.model_mixins import UniqueSubjectIdentifierFieldMixin
from edc_model.models import BaseUuidModel, HistoricalRecords
from edc_sites.models import CurrentSiteManager, SiteModelMixin
from edc_utils import get_utcnow
from edc_visit_tracking.managers import CrfModelManager

from .model_mixins import (
    BiosynexSemiQuantitativeCragMixin,
    CsfCultureModelMixin,
    CsfModelMixin,
    LpModelMixin,
    QuantitativeCultureModelMixin,
)


class LpCsf(
    UniqueSubjectIdentifierFieldMixin,
    LpModelMixin,
    CsfModelMixin,
    CsfCultureModelMixin,
    BiosynexSemiQuantitativeCragMixin,
    QuantitativeCultureModelMixin,
    SiteModelMixin,
    BaseUuidModel,
):
    report_datetime = models.DateTimeField(default=get_utcnow)

    objects = CrfModelManager()

    on_site = CurrentSiteManager()

    history = HistoricalRecords()

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Lumbar Puncture/Cerebrospinal Fluid"
        verbose_name_plural = "Lumbar Puncture/Cerebrospinal Fluid"



from .biosynex_semi_quantitative_crag_mixin import BiosynexSemiQuantitativeCragMixin
from .csf import CsfModelMixin
from .csf_culture import CsfCultureModelMixin
from .lp import LpModelMixin
from .quantitative_culture import QuantitativeCultureModelMixin
